@@ -14,9 +14,9 @@
 #
 
 import copy
-import mock
 
 from glanceclient.v2 import schemas
+import mock
 from osc_lib import exceptions
 from osc_lib import utils as common_utils
 import warlock
@@ -130,7 +130,7 @@ class TestImageCreate(TestImage):
         self.images_mock.configure_mock(**mock_exception)
         arglist = [
             '--container-format', 'ovf',
-            '--disk-format', 'fs',
+            '--disk-format', 'ami',
             '--min-disk', '10',
             '--min-ram', '4',
             ('--protected'
@@ -143,7 +143,7 @@ class TestImageCreate(TestImage):
         ]
         verifylist = [
             ('container_format', 'ovf'),
-            ('disk_format', 'fs'),
+            ('disk_format', 'ami'),
             ('min_disk', 10),
             ('min_ram', 4),
             ('protected', self.new_image.protected),
@@ -165,7 +165,7 @@ class TestImageCreate(TestImage):
         self.images_mock.create.assert_called_with(
             name=self.new_image.name,
             container_format='ovf',
-            disk_format='fs',
+            disk_format='ami',
             min_disk=10,
             min_ram=4,
             owner=self.project.id,
@@ -193,7 +193,7 @@ class TestImageCreate(TestImage):
 
         arglist = [
             '--container-format', 'ovf',
-            '--disk-format', 'fs',
+            '--disk-format', 'ami',
             '--min-disk', '10',
             '--min-ram', '4',
             '--owner', 'unexist_owner',
@@ -203,7 +203,7 @@ class TestImageCreate(TestImage):
         ]
         verifylist = [
             ('container_format', 'ovf'),
-            ('disk_format', 'fs'),
+            ('disk_format', 'ami'),
             ('min_disk', 10),
             ('min_ram', 4),
             ('owner', 'unexist_owner'),
@@ -227,7 +227,7 @@ class TestImageCreate(TestImage):
 
         arglist = [
             '--container-format', 'ovf',
-            '--disk-format', 'fs',
+            '--disk-format', 'ami',
             '--min-disk', '10',
             '--min-ram', '4',
             '--protected',
@@ -237,7 +237,7 @@ class TestImageCreate(TestImage):
         ]
         verifylist = [
             ('container_format', 'ovf'),
-            ('disk_format', 'fs'),
+            ('disk_format', 'ami'),
             ('min_disk', 10),
             ('min_ram', 4),
             ('protected', True),
@@ -535,7 +535,9 @@ class TestImageList(TestImage):
         # returns a tuple containing the column names and an iterable
         # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
-        self.api_mock.image_list.assert_called_with()
+        self.api_mock.image_list.assert_called_with(
+            marker=self._image.id,
+        )
 
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
@@ -558,6 +560,7 @@ class TestImageList(TestImage):
         columns, data = self.cmd.take_action(parsed_args)
         self.api_mock.image_list.assert_called_with(
             public=True,
+            marker=self._image.id,
         )
 
         self.assertEqual(self.columns, columns)
@@ -581,6 +584,7 @@ class TestImageList(TestImage):
         columns, data = self.cmd.take_action(parsed_args)
         self.api_mock.image_list.assert_called_with(
             private=True,
+            marker=self._image.id,
         )
 
         self.assertEqual(self.columns, columns)
@@ -604,6 +608,7 @@ class TestImageList(TestImage):
         columns, data = self.cmd.take_action(parsed_args)
         self.api_mock.image_list.assert_called_with(
             shared=True,
+            marker=self._image.id,
         )
 
         self.assertEqual(self.columns, columns)
@@ -622,7 +627,9 @@ class TestImageList(TestImage):
         # returns a tuple containing the column names and an iterable
         # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
-        self.api_mock.image_list.assert_called_with()
+        self.api_mock.image_list.assert_called_with(
+            marker=self._image.id,
+        )
 
         collist = (
             'ID',
@@ -670,7 +677,9 @@ class TestImageList(TestImage):
         # returns a tuple containing the column names and an iterable
         # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
-        self.api_mock.image_list.assert_called_with()
+        self.api_mock.image_list.assert_called_with(
+            marker=self._image.id,
+        )
         sf_mock.assert_called_with(
             [self._image],
             attr='a',
@@ -693,7 +702,9 @@ class TestImageList(TestImage):
         # returns a tuple containing the column names and an iterable
         # containing the data to be listed.
         columns, data = self.cmd.take_action(parsed_args)
-        self.api_mock.image_list.assert_called_with()
+        self.api_mock.image_list.assert_called_with(
+            marker=self._image.id,
+        )
         si_mock.assert_called_with(
             [self._image],
             'name:asc'
@@ -712,7 +723,7 @@ class TestImageList(TestImage):
 
         columns, data = self.cmd.take_action(parsed_args)
         self.api_mock.image_list.assert_called_with(
-            limit=1,
+            limit=1, marker=self._image.id
         )
 
         self.assertEqual(self.columns, columns)
@@ -739,6 +750,34 @@ class TestImageList(TestImage):
             marker=image_fakes.image_id,
         )
 
+    def test_image_list_name_option(self):
+        arglist = [
+            '--name', 'abc',
+        ]
+        verifylist = [
+            ('name', 'abc'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            name='abc', marker=self._image.id
+        )
+
+    def test_image_list_status_option(self):
+        arglist = [
+            '--status', 'active',
+        ]
+        verifylist = [
+            ('status', 'active'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            status='active', marker=self._image.id
+        )
+
 
 class TestRemoveProjectImage(TestImage):
 
@@ -818,6 +857,11 @@ class TestImageSet(TestImage):
 
         self.images_mock.get.return_value = self.model(**image_fakes.IMAGE)
         self.images_mock.update.return_value = self.model(**image_fakes.IMAGE)
+
+        self.app.client_manager.auth_ref = mock.Mock(
+            project_id=self.project.id,
+        )
+
         # Get the command object to test
         self.cmd = image.SetImage(self.app, None)
 
@@ -834,6 +878,101 @@ class TestImageSet(TestImage):
 
         self.assertIsNone(result)
 
+        self.image_members_mock.update.assert_not_called()
+
+    def test_image_set_membership_option_accept(self):
+        membership = image_fakes.FakeImage.create_one_image_member(
+            attrs={'image_id': image_fakes.image_id,
+                   'member_id': self.project.id}
+        )
+        self.image_members_mock.update.return_value = membership
+
+        arglist = [
+            '--accept',
+            image_fakes.image_id,
+        ]
+        verifylist = [
+            ('accept', True),
+            ('reject', False),
+            ('pending', False),
+            ('image', image_fakes.image_id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.image_members_mock.update.assert_called_once_with(
+            image_fakes.image_id,
+            self.app.client_manager.auth_ref.project_id,
+            'accepted',
+        )
+
+        # Assert that the 'update image" route is also called, in addition to
+        # the 'update membership' route.
+        self.images_mock.update.assert_called_with(image_fakes.image_id)
+
+    def test_image_set_membership_option_reject(self):
+        membership = image_fakes.FakeImage.create_one_image_member(
+            attrs={'image_id': image_fakes.image_id,
+                   'member_id': self.project.id}
+        )
+        self.image_members_mock.update.return_value = membership
+
+        arglist = [
+            '--reject',
+            image_fakes.image_id,
+        ]
+        verifylist = [
+            ('accept', False),
+            ('reject', True),
+            ('pending', False),
+            ('image', image_fakes.image_id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.image_members_mock.update.assert_called_once_with(
+            image_fakes.image_id,
+            self.app.client_manager.auth_ref.project_id,
+            'rejected',
+        )
+
+        # Assert that the 'update image" route is also called, in addition to
+        # the 'update membership' route.
+        self.images_mock.update.assert_called_with(image_fakes.image_id)
+
+    def test_image_set_membership_option_pending(self):
+        membership = image_fakes.FakeImage.create_one_image_member(
+            attrs={'image_id': image_fakes.image_id,
+                   'member_id': self.project.id}
+        )
+        self.image_members_mock.update.return_value = membership
+
+        arglist = [
+            '--pending',
+            image_fakes.image_id,
+        ]
+        verifylist = [
+            ('accept', False),
+            ('reject', False),
+            ('pending', True),
+            ('image', image_fakes.image_id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        self.cmd.take_action(parsed_args)
+
+        self.image_members_mock.update.assert_called_once_with(
+            image_fakes.image_id,
+            self.app.client_manager.auth_ref.project_id,
+            'pending',
+        )
+
+        # Assert that the 'update image" route is also called, in addition to
+        # the 'update membership' route.
+        self.images_mock.update.assert_called_with(image_fakes.image_id)
+
     def test_image_set_options(self):
         arglist = [
             '--name', 'new-name',
@@ -1174,9 +1313,38 @@ class TestImageSet(TestImage):
             exceptions.CommandError,
             self.cmd.take_action, parsed_args)
 
+    def test_image_set_numeric_options_to_zero(self):
+        arglist = [
+            '--min-disk', '0',
+            '--min-ram', '0',
+            image_fakes.image_name,
+        ]
+        verifylist = [
+            ('min_disk', 0),
+            ('min_ram', 0),
+            ('image', image_fakes.image_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        kwargs = {
+            'min_disk': 0,
+            'min_ram': 0,
+        }
+        # ImageManager.update(image, **kwargs)
+        self.images_mock.update.assert_called_with(
+            image_fakes.image_id,
+            **kwargs
+        )
+        self.assertIsNone(result)
+
 
 class TestImageShow(TestImage):
 
+    new_image = image_fakes.FakeImage.create_one_image(
+        attrs={'size': 1000})
+
     def setUp(self):
         super(TestImageShow, self).setUp()
 
@@ -1211,6 +1379,29 @@ class TestImageShow(TestImage):
         self.assertEqual(image_fakes.IMAGE_columns, columns)
         self.assertEqual(image_fakes.IMAGE_SHOW_data, data)
 
+    def test_image_show_human_readable(self):
+        self.images_mock.get.return_value = self.new_image
+        arglist = [
+            '--human-readable',
+            self.new_image.id,
+        ]
+        verifylist = [
+            ('human_readable', True),
+            ('image', self.new_image.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class ShowOne in cliff, abstract method take_action()
+        # returns a two-part tuple with a tuple of column names and a tuple of
+        # data to be shown.
+        columns, data = self.cmd.take_action(parsed_args)
+        self.images_mock.get.assert_called_with(
+            self.new_image.id,
+        )
+
+        size_index = columns.index('size')
+        self.assertEqual(data[size_index], '1K')
+
 
 class TestImageUnset(TestImage):
 
