@@ -13,11 +13,12 @@
 
 import argparse
 import copy
-import mock
 from random import choice
 from random import randint
 import uuid
 
+import mock
+
 from openstackclient.tests.unit import fakes
 from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes_v3
 from openstackclient.tests.unit import utils
@@ -144,7 +145,7 @@ class FakeAddressScope(object):
         If address scopes list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List address scopes:
+        :param List address_scopes:
             A list of FakeResource objects faking address scopes
         :param int count:
             The number of address scopes to fake
@@ -346,9 +347,11 @@ class FakeNetwork(object):
             'availability_zone_hints': [],
             'is_default': False,
             'port_security_enabled': True,
+            'tags': ['test'],
             'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
             'ipv4_address_scope': 'ipv4' + uuid.uuid4().hex,
             'ipv6_address_scope': 'ipv6' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -365,6 +368,7 @@ class FakeNetwork(object):
             network_attrs['port_security_enabled']
         network.subnet_ids = network_attrs['subnets']
         network.is_shared = network_attrs['shared']
+        network.is_tags = network_attrs['tags']
         network.provider_network_type = \
             network_attrs['provider:network_type']
         network.provider_physical_network = \
@@ -556,6 +560,7 @@ class FakePort(object):
             'binding:vif_details': {},
             'binding:vif_type': 'ovs',
             'binding:vnic_type': 'normal',
+            'data_plane_status': None,
             'description': 'description-' + uuid.uuid4().hex,
             'device_id': 'device-id-' + uuid.uuid4().hex,
             'device_owner': 'compute:nova',
@@ -573,6 +578,7 @@ class FakePort(object):
             'status': 'ACTIVE',
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -839,6 +845,7 @@ class FakeNetworkQosPolicy(object):
         qos_policy_attrs = {
             'name': 'qos-policy-name-' + uuid.uuid4().hex,
             'id': qos_id,
+            'is_default': False,
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
             'shared': False,
             'description': 'qos-policy-description-' + uuid.uuid4().hex,
@@ -883,7 +890,7 @@ class FakeNetworkQosPolicy(object):
         If qos policies list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List address scopes:
+        :param List qos_policies:
             A list of FakeResource objects faking qos policies
         :param int count:
             The number of QoS policies to fake
@@ -921,6 +928,7 @@ class FakeNetworkQosRule(object):
         if type == RULE_TYPE_BANDWIDTH_LIMIT:
             qos_rule_attrs['max_kbps'] = randint(1, 10000)
             qos_rule_attrs['max_burst_kbits'] = randint(1, 10000)
+            qos_rule_attrs['direction'] = 'egress'
         elif type == RULE_TYPE_DSCP_MARKING:
             qos_rule_attrs['dscp_mark'] = choice(VALID_DSCP_MARKS)
         elif type == RULE_TYPE_MINIMUM_BANDWIDTH:
@@ -961,7 +969,7 @@ class FakeNetworkQosRule(object):
         If Network QoS rules list is provided, then initialize the Mock
         object with the list. Otherwise create one.
 
-        :param List address scopes:
+        :param List qos_rules:
             A list of FakeResource objects faking Network QoS rules
         :param int count:
             The number of QoS minimum bandwidth rules to fake
@@ -1048,6 +1056,7 @@ class FakeRouter(object):
             'external_gateway_info': {},
             'availability_zone_hints': [],
             'availability_zones': [],
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -1161,7 +1170,7 @@ class FakeSecurityGroup(object):
         If security groups list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List security groups:
+        :param List security_groups:
             A list of FakeResource objects faking security groups
         :param int count:
             The number of security groups to fake
@@ -1241,7 +1250,7 @@ class FakeSecurityGroupRule(object):
         If security group rules list is provided, then initialize the Mock
         object with the list. Otherwise create one.
 
-        :param List security group rules:
+        :param List security_group_rules:
             A list of FakeResource objects faking security group rules
         :param int count:
             The number of security group rules to fake
@@ -1289,6 +1298,7 @@ class FakeSubnet(object):
             'service_types': [],
             'subnetpool_id': None,
             'description': 'subnet-description-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -1406,7 +1416,7 @@ class FakeFloatingIP(object):
         If floating_ips list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List floating ips:
+        :param List floating_ips:
             A list of FakeResource objects faking floating ips
         :param int count:
             The number of floating ips to fake
@@ -1539,6 +1549,7 @@ class FakeSubnetPool(object):
             'default_quota': None,
             'ip_version': '4',
             'description': 'subnet-pool-description-' + uuid.uuid4().hex,
+            'tags': [],
         }
 
         # Overwrite default attributes.
@@ -1585,7 +1596,7 @@ class FakeSubnetPool(object):
         If subnet_pools list is provided, then initialize the Mock object
         with the list. Otherwise create one.
 
-        :param List subnet pools:
+        :param List subnet_pools:
             A list of FakeResource objects faking subnet pools
         :param int count:
             The number of subnet pools to fake
