@@ -12,8 +12,8 @@
 #
 
 import argparse
-import mock
 
+import mock
 from mock import call
 from osc_lib import exceptions
 from osc_lib import utils
@@ -35,7 +35,8 @@ class TestPort(network_fakes.TestNetworkV2):
         # Get a shortcut to the ProjectManager Mock
         self.projects_mock = self.app.client_manager.identity.projects
 
-    def _get_common_cols_data(self, fake_port):
+    @staticmethod
+    def _get_common_cols_data(fake_port):
         columns = (
             'admin_state_up',
             'allowed_address_pairs',
@@ -44,6 +45,7 @@ class TestPort(network_fakes.TestNetworkV2):
             'binding_vif_details',
             'binding_vif_type',
             'binding_vnic_type',
+            'data_plane_status',
             'description',
             'device_id',
             'device_owner',
@@ -57,8 +59,10 @@ class TestPort(network_fakes.TestNetworkV2):
             'network_id',
             'port_security_enabled',
             'project_id',
+            'qos_policy_id',
             'security_group_ids',
             'status',
+            'tags',
         )
 
         data = (
@@ -69,6 +73,7 @@ class TestPort(network_fakes.TestNetworkV2):
             utils.format_dict(fake_port.binding_vif_details),
             fake_port.binding_vif_type,
             fake_port.binding_vnic_type,
+            fake_port.data_plane_status,
             fake_port.description,
             fake_port.device_id,
             fake_port.device_owner,
@@ -82,8 +87,10 @@ class TestPort(network_fakes.TestNetworkV2):
             fake_port.network_id,
             fake_port.port_security_enabled,
             fake_port.project_id,
+            fake_port.qos_policy_id,
             utils.format_list(fake_port.security_group_ids),
             fake_port.status,
+            utils.format_list(fake_port.tags),
         )
 
         return columns, data
@@ -92,11 +99,13 @@ class TestPort(network_fakes.TestNetworkV2):
 class TestCreatePort(TestPort):
 
     _port = network_fakes.FakePort.create_one_port()
+    columns, data = TestPort._get_common_cols_data(_port)
 
     def setUp(self):
         super(TestCreatePort, self).setUp()
 
         self.network.create_port = mock.Mock(return_value=self._port)
+        self.network.set_tags = mock.Mock(return_value=None)
         fake_net = network_fakes.FakeNetwork.create_one_network({
             'id': self._port.network_id,
         })
@@ -125,10 +134,10 @@ class TestCreatePort(TestPort):
             'network_id': self._port.network_id,
             'name': 'test-port',
         })
+        self.assertFalse(self.network.set_tags.called)
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_full_options(self):
         arglist = [
@@ -162,7 +171,6 @@ class TestCreatePort(TestPort):
             ('network', self._port.network_id),
             ('dns_name', '8.8.8.8'),
             ('name', 'test-port'),
-
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -183,9 +191,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_invalid_json_binding_profile(self):
         arglist = [
@@ -235,9 +242,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_with_security_group(self):
         secgroup = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -265,9 +271,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_dns_name(self):
         arglist = [
@@ -292,9 +297,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_with_security_groups(self):
         sg_1 = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -323,9 +327,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_with_no_security_groups(self):
         arglist = [
@@ -350,9 +353,34 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_no_fixed_ips(self):
+        arglist = [
+            '--network', self._port.network_id,
+            '--no-fixed-ip',
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id),
+            ('enable', True),
+            ('no_fixed_ip', True),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'fixed_ips': [],
+            'name': 'test-port',
+        })
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_allowed_address_pair_ipaddr(self):
         pairs = [{'ip_address': '192.168.1.123'},
@@ -381,9 +409,8 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_with_allowed_address_pair(self):
         pairs = [{'ip_address': '192.168.1.123',
@@ -418,9 +445,36 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_port_with_qos(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        arglist = [
+            '--network', self._port.network_id,
+            '--qos-policy', qos_policy.id,
+            'test-port',
+        ]
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('qos_policy', qos_policy.id),
+            ('name', 'test-port'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(**{
+            'admin_state_up': True,
+            'network_id': self._port.network_id,
+            'qos_policy_id': qos_policy.id,
+            'name': 'test-port',
+        })
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
     def test_create_port_security_enabled(self):
         arglist = [
@@ -470,6 +524,48 @@ class TestCreatePort(TestPort):
             'name': 'test-port',
         })
 
+    def _test_create_with_tag(self, add_tags=True):
+        arglist = [
+            '--network', self._port.network_id,
+            'test-port',
+        ]
+        if add_tags:
+            arglist += ['--tag', 'red', '--tag', 'blue']
+        else:
+            arglist += ['--no-tag']
+        verifylist = [
+            ('network', self._port.network_id,),
+            ('enable', True),
+            ('name', 'test-port'),
+        ]
+        if add_tags:
+            verifylist.append(('tags', ['red', 'blue']))
+        else:
+            verifylist.append(('no_tag', True))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = (self.cmd.take_action(parsed_args))
+
+        self.network.create_port.assert_called_once_with(
+            admin_state_up=True,
+            network_id=self._port.network_id,
+            name='test-port'
+        )
+        if add_tags:
+            self.network.set_tags.assert_called_once_with(
+                self._port,
+                tests_utils.CompareBySet(['red', 'blue']))
+        else:
+            self.assertFalse(self.network.set_tags.called)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
+
+    def test_create_with_tags(self):
+        self._test_create_with_tag(add_tags=True)
+
+    def test_create_with_no_tag(self):
+        self._test_create_with_tag(add_tags=False)
+
 
 class TestDeletePort(TestPort):
 
@@ -570,6 +666,7 @@ class TestListPort(TestPort):
         'Status',
         'Security Groups',
         'Device Owner',
+        'Tags',
     )
 
     data = []
@@ -592,6 +689,7 @@ class TestListPort(TestPort):
             prt.status,
             utils.format_list(prt.security_group_ids),
             prt.device_owner,
+            utils.format_list(prt.tags),
         ))
 
     def setUp(self):
@@ -868,10 +966,35 @@ class TestListPort(TestPort):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.data, list(data))
 
+    def test_list_with_tag_options(self):
+        arglist = [
+            '--tags', 'red,blue',
+            '--any-tags', 'red,green',
+            '--not-tags', 'orange,yellow',
+            '--not-any-tags', 'black,white',
+        ]
+        verifylist = [
+            ('tags', ['red', 'blue']),
+            ('any_tags', ['red', 'green']),
+            ('not_tags', ['orange', 'yellow']),
+            ('not_any_tags', ['black', 'white']),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.network.ports.assert_called_once_with(
+            **{'tags': 'red,blue',
+               'any_tags': 'red,green',
+               'not_tags': 'orange,yellow',
+               'not_any_tags': 'black,white'}
+        )
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, list(data))
+
 
 class TestSetPort(TestPort):
 
-    _port = network_fakes.FakePort.create_one_port()
+    _port = network_fakes.FakePort.create_one_port({'tags': ['green', 'red']})
 
     def setUp(self):
         super(TestSetPort, self).setUp()
@@ -879,31 +1002,74 @@ class TestSetPort(TestPort):
         self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
         self.network.find_port = mock.Mock(return_value=self._port)
         self.network.update_port = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
 
         # Get the command object to test
         self.cmd = port.SetPort(self.app, self.namespace)
 
-    def test_set_fixed_ip(self):
+    def test_set_port_defaults(self):
         arglist = [
-            '--fixed-ip', 'ip-address=10.0.0.11',
             self._port.name,
-            '--no-fixed-ip',
         ]
         verifylist = [
-            ('fixed_ip', [{'ip-address': '10.0.0.11'}]),
             ('port', self._port.name),
         ]
-
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
 
-        attrs = {
-            'fixed_ips': [{'ip_address': '10.0.0.11'}],
-        }
-        self.network.update_port.assert_called_once_with(self._port, **attrs)
+        result = self.cmd.take_action(parsed_args)
+        self.assertFalse(self.network.update_port.called)
+        self.assertFalse(self.network.set_tags.called)
         self.assertIsNone(result)
 
-    def test_set_dns_name(self):
+    def test_set_port_fixed_ip(self):
+        _testport = network_fakes.FakePort.create_one_port(
+            {'fixed_ips': [{'ip_address': '0.0.0.1'}]})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--fixed-ip', 'ip-address=10.0.0.12',
+            _testport.name,
+        ]
+        verifylist = [
+            ('fixed_ip', [{'ip-address': '10.0.0.12'}]),
+            ('port', _testport.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'fixed_ips': [
+                {'ip_address': '0.0.0.1'},
+                {'ip_address': '10.0.0.12'},
+            ],
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_port_fixed_ip_clear(self):
+        _testport = network_fakes.FakePort.create_one_port(
+            {'fixed_ips': [{'ip_address': '0.0.0.1'}]})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--fixed-ip', 'ip-address=10.0.0.12',
+            '--no-fixed-ip',
+            _testport.name,
+        ]
+        verifylist = [
+            ('fixed_ip', [{'ip-address': '10.0.0.12'}]),
+            ('no_fixed_ip', True)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        attrs = {
+            'fixed_ips': [
+                {'ip_address': '10.0.0.12'},
+            ],
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_port_dns_name(self):
         arglist = [
             '--dns-name', '8.8.8.8',
             self._port.name,
@@ -922,28 +1088,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_append_fixed_ip(self):
-        _testport = network_fakes.FakePort.create_one_port(
-            {'fixed_ips': [{'ip_address': '0.0.0.1'}]})
-        self.network.find_port = mock.Mock(return_value=_testport)
-        arglist = [
-            '--fixed-ip', 'ip-address=10.0.0.12',
-            _testport.name,
-        ]
-        verifylist = [
-            ('fixed_ip', [{'ip-address': '10.0.0.12'}]),
-            ('port', _testport.name),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-        attrs = {
-            'fixed_ips': [
-                {'ip_address': '10.0.0.12'}, {'ip_address': '0.0.0.1'}],
-        }
-        self.network.update_port.assert_called_once_with(_testport, **attrs)
-        self.assertIsNone(result)
-
-    def test_overwrite_binding_profile(self):
+    def test_set_port_overwrite_binding_profile(self):
         _testport = network_fakes.FakePort.create_one_port(
             {'binding_profile': {'lok_i': 'visi_on'}})
         self.network.find_port = mock.Mock(return_value=_testport)
@@ -965,29 +1110,26 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
-    def test_overwrite_fixed_ip(self):
+    def test_overwrite_mac_address(self):
         _testport = network_fakes.FakePort.create_one_port(
-            {'fixed_ips': [{'ip_address': '0.0.0.1'}]})
+            {'mac_address': '11:22:33:44:55:66'})
         self.network.find_port = mock.Mock(return_value=_testport)
         arglist = [
-            '--fixed-ip', 'ip-address=10.0.0.12',
-            '--no-fixed-ip',
+            '--mac-address', '66:55:44:33:22:11',
             _testport.name,
         ]
         verifylist = [
-            ('fixed_ip', [{'ip-address': '10.0.0.12'}]),
-            ('no_fixed_ip', True)
+            ('mac_address', '66:55:44:33:22:11'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
         attrs = {
-            'fixed_ips': [
-                {'ip_address': '10.0.0.12'}],
+            'mac_address': '66:55:44:33:22:11',
         }
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
-    def test_set_this(self):
+    def test_set_port_this(self):
         arglist = [
             '--disable',
             '--no-fixed-ip',
@@ -1012,7 +1154,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_set_that(self):
+    def test_set_port_that(self):
         arglist = [
             '--description', 'newDescription',
             '--enable',
@@ -1046,22 +1188,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_set_nothing(self):
-        arglist = [
-            self._port.name,
-        ]
-        verifylist = [
-            ('port', self._port.name),
-        ]
-
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
-
-        attrs = {}
-        self.network.update_port.assert_called_once_with(self._port, **attrs)
-        self.assertIsNone(result)
-
-    def test_set_invalid_json_binding_profile(self):
+    def test_set_port_invalid_json_binding_profile(self):
         arglist = [
             '--binding-profile', '{"parent_name"}',
             'test-port',
@@ -1072,7 +1199,7 @@ class TestSetPort(TestPort):
                           arglist,
                           None)
 
-    def test_set_invalid_key_value_binding_profile(self):
+    def test_set_port_invalid_key_value_binding_profile(self):
         arglist = [
             '--binding-profile', 'key',
             'test-port',
@@ -1083,7 +1210,7 @@ class TestSetPort(TestPort):
                           arglist,
                           None)
 
-    def test_set_mixed_binding_profile(self):
+    def test_set_port_mixed_binding_profile(self):
         arglist = [
             '--binding-profile', 'foo=bar',
             '--binding-profile', '{"foo2": "bar2"}',
@@ -1103,7 +1230,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_set_security_group(self):
+    def test_set_port_security_group(self):
         sg = network_fakes.FakeSecurityGroup.create_one_security_group()
         self.network.find_security_group = mock.Mock(return_value=sg)
         arglist = [
@@ -1114,17 +1241,16 @@ class TestSetPort(TestPort):
             ('security_group', [sg.id]),
             ('port', self._port.name),
         ]
-
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
 
+        result = self.cmd.take_action(parsed_args)
         attrs = {
             'security_group_ids': [sg.id],
         }
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_append_security_group(self):
+    def test_set_port_security_group_append(self):
         sg_1 = network_fakes.FakeSecurityGroup.create_one_security_group()
         sg_2 = network_fakes.FakeSecurityGroup.create_one_security_group()
         sg_3 = network_fakes.FakeSecurityGroup.create_one_security_group()
@@ -1142,14 +1268,15 @@ class TestSetPort(TestPort):
             ('port', _testport.name),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
         result = self.cmd.take_action(parsed_args)
         attrs = {
-            'security_group_ids': [sg_2.id, sg_3.id, sg_1.id],
+            'security_group_ids': [sg_1.id, sg_2.id, sg_3.id],
         }
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
-    def test_set_no_security_groups(self):
+    def test_set_port_security_group_clear(self):
         arglist = [
             '--no-security-group',
             self._port.name,
@@ -1158,17 +1285,16 @@ class TestSetPort(TestPort):
             ('no_security_group', True),
             ('port', self._port.name),
         ]
-
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-        result = self.cmd.take_action(parsed_args)
 
+        result = self.cmd.take_action(parsed_args)
         attrs = {
             'security_group_ids': [],
         }
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_overwrite_security_group(self):
+    def test_set_port_security_group_replace(self):
         sg1 = network_fakes.FakeSecurityGroup.create_one_security_group()
         sg2 = network_fakes.FakeSecurityGroup.create_one_security_group()
         _testport = network_fakes.FakePort.create_one_port(
@@ -1185,6 +1311,7 @@ class TestSetPort(TestPort):
             ('no_security_group', True)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
         result = self.cmd.take_action(parsed_args)
         attrs = {
             'security_group_ids': [sg2.id],
@@ -1192,7 +1319,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
-    def test_set_allowed_address_pair(self):
+    def test_set_port_allowed_address_pair(self):
         arglist = [
             '--allowed-address', 'ip-address=192.168.1.123',
             self._port.name,
@@ -1211,7 +1338,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_append_allowed_address_pair(self):
+    def test_set_port_append_allowed_address_pair(self):
         _testport = network_fakes.FakePort.create_one_port(
             {'allowed_address_pairs': [{'ip_address': '192.168.1.123'}]})
         self.network.find_port = mock.Mock(return_value=_testport)
@@ -1234,7 +1361,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
-    def test_overwrite_allowed_address_pair(self):
+    def test_set_port_overwrite_allowed_address_pair(self):
         _testport = network_fakes.FakePort.create_one_port(
             {'allowed_address_pairs': [{'ip_address': '192.168.1.123'}]})
         self.network.find_port = mock.Mock(return_value=_testport)
@@ -1258,7 +1385,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(_testport, **attrs)
         self.assertIsNone(result)
 
-    def test_set_no_allowed_address_pairs(self):
+    def test_set_port_no_allowed_address_pairs(self):
         arglist = [
             '--no-allowed-address',
             self._port.name,
@@ -1277,7 +1404,7 @@ class TestSetPort(TestPort):
         self.network.update_port.assert_called_once_with(self._port, **attrs)
         self.assertIsNone(result)
 
-    def test_port_security_enabled(self):
+    def test_set_port_security_enabled(self):
         arglist = [
             '--enable-port-security',
             self._port.id,
@@ -1295,7 +1422,7 @@ class TestSetPort(TestPort):
             'port_security_enabled': True,
         })
 
-    def test_port_security_disabled(self):
+    def test_set_port_security_disabled(self):
         arglist = [
             '--disable-port-security',
             self._port.id,
@@ -1313,11 +1440,98 @@ class TestSetPort(TestPort):
             'port_security_enabled': False,
         })
 
+    def test_set_port_with_qos(self):
+        qos_policy = network_fakes.FakeNetworkQosPolicy.create_one_qos_policy()
+        self.network.find_qos_policy = mock.Mock(return_value=qos_policy)
+        _testport = network_fakes.FakePort.create_one_port(
+            {'qos_policy_id': None})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--qos-policy', qos_policy.id,
+            _testport.name,
+        ]
+        verifylist = [
+            ('qos_policy', qos_policy.id),
+            ('port', _testport.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'qos_policy_id': qos_policy.id,
+        }
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_port_data_plane_status(self):
+        _testport = network_fakes.FakePort.create_one_port(
+            {'data_plane_status': None})
+        self.network.find_port = mock.Mock(return_value=_testport)
+        arglist = [
+            '--data-plane-status', 'ACTIVE',
+            _testport.name,
+        ]
+        verifylist = [
+            ('data_plane_status', 'ACTIVE'),
+            ('port', _testport.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'data_plane_status': 'ACTIVE',
+        }
+
+        self.network.update_port.assert_called_once_with(_testport, **attrs)
+        self.assertIsNone(result)
+
+    def test_set_port_invalid_data_plane_status_value(self):
+        arglist = [
+            '--data-plane-status', 'Spider-Man',
+            'test-port',
+        ]
+        self.assertRaises(tests_utils.ParserException,
+                          self.check_parser,
+                          self.cmd,
+                          arglist,
+                          None)
+
+    def _test_set_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['red', 'blue', 'green']
+        else:
+            arglist = ['--no-tag']
+            verifylist = [('no_tag', True)]
+            expected_args = []
+        arglist.append(self._port.name)
+        verifylist.append(
+            ('port', self._port.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_port.called)
+        self.network.set_tags.assert_called_once_with(
+            self._port,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_set_with_tags(self):
+        self._test_set_tags(with_tags=True)
+
+    def test_set_with_no_tag(self):
+        self._test_set_tags(with_tags=False)
+
 
 class TestShowPort(TestPort):
 
     # The port to show.
     _port = network_fakes.FakePort.create_one_port()
+    columns, data = TestPort._get_common_cols_data(_port)
 
     def setUp(self):
         super(TestShowPort, self).setUp()
@@ -1348,9 +1562,8 @@ class TestShowPort(TestPort):
         self.network.find_port.assert_called_once_with(
             self._port.name, ignore_missing=False)
 
-        ref_columns, ref_data = self._get_common_cols_data(self._port)
-        self.assertEqual(ref_columns, columns)
-        self.assertEqual(ref_data, data)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.data, data)
 
 
 class TestUnsetPort(TestPort):
@@ -1362,12 +1575,14 @@ class TestUnsetPort(TestPort):
                             'ip_address': '0.0.0.1'},
                            {'subnet_id': '042eb10a-3a18-4658-ab-cf47c8d03152',
                             'ip_address': '1.0.0.0'}],
-             'binding:profile': {'batman': 'Joker', 'Superman': 'LexLuthor'}})
+             'binding:profile': {'batman': 'Joker', 'Superman': 'LexLuthor'},
+             'tags': ['green', 'red'], })
         self.fake_subnet = network_fakes.FakeSubnet.create_one_subnet(
             {'id': '042eb10a-3a18-4658-ab-cf47c8d03152'})
         self.network.find_subnet = mock.Mock(return_value=self.fake_subnet)
         self.network.find_port = mock.Mock(return_value=self._testport)
         self.network.update_port = mock.Mock(return_value=None)
+        self.network.set_tags = mock.Mock(return_value=None)
         # Get the command object to test
         self.cmd = port.UnsetPort(self.app, self.namespace)
 
@@ -1376,6 +1591,7 @@ class TestUnsetPort(TestPort):
             '--fixed-ip',
             'subnet=042eb10a-3a18-4658-ab-cf47c8d03152,ip-address=1.0.0.0',
             '--binding-profile', 'Superman',
+            '--qos-policy',
             self._testport.name,
         ]
         verifylist = [
@@ -1383,6 +1599,7 @@ class TestUnsetPort(TestPort):
                 'subnet': '042eb10a-3a18-4658-ab-cf47c8d03152',
                 'ip-address': '1.0.0.0'}]),
             ('binding_profile', ['Superman']),
+            ('qos_policy', True),
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -1392,7 +1609,8 @@ class TestUnsetPort(TestPort):
             'fixed_ips': [{
                 'subnet_id': '042eb10a-3a18-4658-ab-cf47c8d03152',
                 'ip_address': '0.0.0.1'}],
-            'binding:profile': {'batman': 'Joker'}
+            'binding:profile': {'batman': 'Joker'},
+            'qos_policy_id': None
         }
         self.network.update_port.assert_called_once_with(
             self._testport, **attrs)
@@ -1512,3 +1730,54 @@ class TestUnsetPort(TestPort):
         self.assertRaises(exceptions.CommandError,
                           self.cmd.take_action,
                           parsed_args)
+
+    def test_unset_port_data_plane_status(self):
+        _fake_port = network_fakes.FakePort.create_one_port(
+            {'data_plane_status': 'ACTIVE'})
+        self.network.find_port = mock.Mock(return_value=_fake_port)
+        arglist = [
+            '--data-plane-status',
+            _fake_port.name,
+        ]
+        verifylist = [
+            ('data_plane_status', True),
+            ('port', _fake_port.name),
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        attrs = {
+            'data_plane_status': None,
+        }
+
+        self.network.update_port.assert_called_once_with(_fake_port, **attrs)
+        self.assertIsNone(result)
+
+    def _test_unset_tags(self, with_tags=True):
+        if with_tags:
+            arglist = ['--tag', 'red', '--tag', 'blue']
+            verifylist = [('tags', ['red', 'blue'])]
+            expected_args = ['green']
+        else:
+            arglist = ['--all-tag']
+            verifylist = [('all_tag', True)]
+            expected_args = []
+        arglist.append(self._testport.name)
+        verifylist.append(
+            ('port', self._testport.name))
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        result = self.cmd.take_action(parsed_args)
+
+        self.assertFalse(self.network.update_port.called)
+        self.network.set_tags.assert_called_once_with(
+            self._testport,
+            tests_utils.CompareBySet(expected_args))
+        self.assertIsNone(result)
+
+    def test_unset_with_tags(self):
+        self._test_unset_tags(with_tags=True)
+
+    def test_unset_with_all_tag(self):
+        self._test_unset_tags(with_tags=False)
